@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  * The Totem project hereby grant permission for non-gpl compatible GStreamer
  * plugins to be used and distributed together with GStreamer and Totem. This
@@ -36,9 +36,6 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifndef HAVE_GTK_ONLY
-#include <libgnomeui/gnome-authentication-manager.h>
-#endif
 #include "bacon-video-widget.h"
 #include "totem-resources.h"
 
@@ -60,6 +57,11 @@ static gboolean g_fatal_warnings = FALSE;
 static gint64 second_index = -1;
 static char **filenames = NULL;
 
+typedef struct {
+	const char *output;
+	const char *input;
+} callback_data;
+
 #ifdef THUMB_DEBUG
 static void
 show_pixbuf (GdkPixbuf *pix)
@@ -329,7 +331,9 @@ save_pixbuf (GdkPixbuf *pixbuf, const char *path,
 }
 
 static GdkPixbuf *
-capture_interesting_frame (BaconVideoWidget * bvw, char *input, char *output) 
+capture_interesting_frame (BaconVideoWidget *bvw,
+			   const char *input,
+			   const char *output) 
 {
 	GdkPixbuf* pixbuf;
 	guint current;
@@ -346,9 +350,9 @@ capture_interesting_frame (BaconVideoWidget * bvw, char *input, char *output)
 	 * interesting frame */
 	for (current = 0; current < G_N_ELEMENTS(frame_locations); current++)
 	{
-		PROGRESS_DEBUG("About to seek to %f\n", frame_locations[current]);
+		PROGRESS_DEBUG("About to seek to %f", frame_locations[current]);
 		if (bacon_video_widget_seek (bvw, frame_locations[current], NULL) == FALSE) {
-			PROGRESS_DEBUG("Couldn't seek to %f\n", frame_locations[current]);
+			PROGRESS_DEBUG("Couldn't seek to %f", frame_locations[current]);
 			bacon_video_widget_play (bvw, NULL);
 		}
 
@@ -365,10 +369,10 @@ capture_interesting_frame (BaconVideoWidget * bvw, char *input, char *output)
 		}
 
 		/* Pull the frame, if it's interesting we bail early */
-		PROGRESS_DEBUG("About to get frame for iter %d\n", current);
+		PROGRESS_DEBUG("About to get frame for iter %d", current);
 		pixbuf = bacon_video_widget_get_current_frame (bvw);
 		if (pixbuf != NULL && is_image_interesting (pixbuf) != FALSE) {
-			PROGRESS_DEBUG("Frame for iter %d is interesting\n", current);
+			PROGRESS_DEBUG("Frame for iter %d is interesting", current);
 			break;
 		}
 
@@ -380,13 +384,16 @@ capture_interesting_frame (BaconVideoWidget * bvw, char *input, char *output)
 				pixbuf = NULL;
 			}
 		}
-		PROGRESS_DEBUG("Frame for iter %d was not interesting\n", current);
+		PROGRESS_DEBUG("Frame for iter %d was not interesting", current);
 	}
 	return pixbuf;
 }
 
 static GdkPixbuf *
-capture_frame_at_time(BaconVideoWidget *bvw, char *input, char *output,  gint64 seconds) 
+capture_frame_at_time(BaconVideoWidget *bvw,
+		      const char *input,
+		      const char *output,
+		      gint64 seconds) 
 {
 	GError *err = NULL;
 
@@ -415,6 +422,42 @@ capture_frame_at_time(BaconVideoWidget *bvw, char *input, char *output,  gint64
 	return bacon_video_widget_get_current_frame (bvw);
 }
 
+static gboolean
+has_audio (BaconVideoWidget *bvw)
+{
+	GValue value = { 0, };
+
+	bacon_video_widget_get_metadata (bvw, BVW_INFO_HAS_VIDEO, &value);
+	return g_value_get_boolean (&value);
+}
+
+static void
+on_got_metadata_event (BaconVideoWidget *bvw, callback_data *data)
+{
+	GValue value = { 0, };
+	GdkPixbuf *pixbuf;
+
+	PROGRESS_DEBUG("Got metadata, checking if we have a cover");
+	bacon_video_widget_get_metadata (bvw, BVW_INFO_COVER, &value);
+	pixbuf = g_value_get_object (&value);
+
+	if (pixbuf) {
+		PROGRESS_DEBUG("Saving cover image");
+
+		bacon_video_widget_close (bvw);
+		totem_resources_monitor_stop ();
+		gtk_widget_destroy (GTK_WIDGET (bvw));
+
+		save_pixbuf (pixbuf, data->output, data->input, output_size, TRUE);
+		g_object_unref (pixbuf);
+
+		exit (0);
+	} else if (has_audio (bvw) == FALSE) {
+		PROGRESS_DEBUG("No covers, and no video, exiting");
+		exit (0);
+	}
+}
+
 static const GOptionEntry entries[] = {
 	{ "jpeg", 'j',  0, G_OPTION_ARG_NONE, &jpeg_output, "Output the thumbnail as a JPEG instead of PNG", NULL },
 	{ "size", 's', 0, G_OPTION_ARG_INT, &output_size, "Size of the thumbnail in pixels", NULL },
@@ -435,7 +478,8 @@ int main (int argc, char *argv[])
 	GError *err = NULL;
 	BaconVideoWidget *bvw;
 	GdkPixbuf *pixbuf;
-	char *input, *output;
+	const char *input, *output;
+	callback_data data;
 
 #ifdef G_OS_UNIX
 	nice (20);
@@ -459,7 +503,6 @@ int main (int argc, char *argv[])
 		return 1;
 	}
 
-#ifndef THUMB_DEBUG
 	if (g_fatal_warnings) {
 		GLogLevelFlags fatal_mask;
 
@@ -467,20 +510,18 @@ int main (int argc, char *argv[])
 		fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
 		g_log_set_always_fatal (fatal_mask);
 	}
-#endif /* THUMB_DEBUG */
-
-#ifndef HAVE_GTK_ONLY
-	gnome_authentication_manager_init ();
-#endif
 
 	if (filenames == NULL || g_strv_length (filenames) != 2) {
-		g_print ("Expects an input and an output file\n");
+		char *help;
+		help = g_option_context_get_help (context, FALSE, NULL);
+		g_print ("%s", help);
+		g_free (help);
 		return 1;
 	}
 	input = filenames[0];
 	output = filenames[1];
 
-	PROGRESS_DEBUG("Initialised libraries, about to create video widget\n");
+	PROGRESS_DEBUG("Initialised libraries, about to create video widget");
 
 	bvw = BACON_VIDEO_WIDGET (bacon_video_widget_new (-1, -1, BVW_USE_TYPE_CAPTURE, &err));
 	if (err != NULL) {
@@ -489,13 +530,18 @@ int main (int argc, char *argv[])
 		g_error_free (err);
 		exit (1);
 	}
+	data.input = input;
+	data.output = output;
+	g_signal_connect (G_OBJECT (bvw), "got-metadata",
+			  G_CALLBACK (on_got_metadata_event),
+			  &data);
 
-	PROGRESS_DEBUG("Video widget created\n");
+	PROGRESS_DEBUG("Video widget created");
 
 	if (time_limit != FALSE)
 		totem_resources_monitor_start (input, 0);
 
-	PROGRESS_DEBUG("About to open video file\n");
+	PROGRESS_DEBUG("About to open video file");
 
 	if (bacon_video_widget_open (bvw, input, &err) == FALSE) {
 		g_print ("totem-video-thumbnailer couldn't open file '%s'\n"
@@ -505,8 +551,8 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	PROGRESS_DEBUG("Opened video file: '%s'\n", input);
-	PROGRESS_DEBUG("About to play file\n");
+	PROGRESS_DEBUG("Opened video file: '%s'", input);
+	PROGRESS_DEBUG("About to play file");
 
 	bacon_video_widget_play (bvw, &err);
 	if (err != NULL) {
@@ -515,8 +561,7 @@ int main (int argc, char *argv[])
 		g_error_free (err);
 		exit (1);
 	}
-
-	PROGRESS_DEBUG("Started playing file\n");
+	PROGRESS_DEBUG("Started playing file");
 
 	/* If the user has told us to use a frame at a specific second 
 	 * into the video, just use that frame no matter how boring it
@@ -537,7 +582,7 @@ int main (int argc, char *argv[])
 		exit (1);
 	}
 
-	PROGRESS_DEBUG("Saving captured screenshot\n");
+	PROGRESS_DEBUG("Saving captured screenshot");
 	save_pixbuf (pixbuf, output, input, output_size, FALSE);
 	g_object_unref (pixbuf);
 
