@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  * The Totem project hereby grant permission for non-gpl compatible GStreamer
  * plugins to be used and distributed together with GStreamer and Totem. This
@@ -121,9 +121,34 @@ long_action (void)
 		gtk_main_iteration ();
 }
 
+static void
+reset_seek_status (Totem *totem)
+{
+	/* Release the lock and reset everything so that we
+	 * avoid being "stuck" seeking on errors */
+
+	if (totem->seek_lock != FALSE) {
+		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), FALSE);
+		totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->fs->time_label), FALSE);
+		totem->seek_lock = FALSE;
+		bacon_video_widget_seek (totem->bvw, 0, NULL);
+		totem_action_stop (totem);
+	}
+}
+
+/**
+ * totem_action_error:
+ * @title: the error dialog title
+ * @reason: the error dialog text
+ * @totem: a #TotemObject
+ *
+ * Displays a non-blocking error dialog with the
+ * given @title and @reason.
+ **/
 void
 totem_action_error (const char *title, const char *reason, Totem *totem)
 {
+	reset_seek_status (totem);
 	totem_interface_error (title, reason,
 			GTK_WINDOW (totem->win));
 }
@@ -132,6 +157,7 @@ static void
 totem_action_error_and_exit (const char *title,
 		const char *reason, Totem *totem)
 {
+	reset_seek_status (totem);
 	totem_interface_error_blocking (title, reason,
 			GTK_WINDOW (totem->win));
 	totem_action_exit (totem);
@@ -199,6 +225,12 @@ totem_action_wait_force_exit (gpointer user_data)
 	exit (1);
 }
 
+/**
+ * totem_action_exit:
+ * @totem: a #TotemObject
+ *
+ * Closes Totem.
+ **/
 void
 totem_action_exit (Totem *totem)
 {
@@ -367,6 +399,13 @@ totem_action_show_properties (Totem *totem)
 		totem_sidebar_set_current_page (totem, "properties", TRUE);
 }
 
+/**
+ * totem_action_play:
+ * @totem: a #TotemObject
+ *
+ * Plays the current stream. If Totem is already playing, it continues
+ * to play. If the stream cannot be played, and error dialog is displayed.
+ **/
 void
 totem_action_play (Totem *totem)
 {
@@ -417,13 +456,7 @@ totem_action_seek (Totem *totem, double pos)
 		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 		g_free (disp);
 
-		/* Release the lock and reset everything so that we
-		 * avoid being "stuck" seeking */
-		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), FALSE);
-		totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->fs->time_label), FALSE);
-		totem->seek_lock = FALSE;
-		bacon_video_widget_seek (totem->bvw, 0, NULL);
-		totem_action_stop (totem);
+		reset_seek_status (totem);
 
 		totem_action_error (msg, err->message, totem);
 		g_free (msg);
@@ -431,6 +464,16 @@ totem_action_seek (Totem *totem, double pos)
 	}
 }
 
+/**
+ * totem_action_set_mrl_and_play:
+ * @totem: a #TotemObject
+ * @mrl: the MRL to play
+ * @subtitle: a subtitle file to load, or %NULL
+ *
+ * Loads the specified @mrl and plays it, if possible.
+ * Calls totem_action_set_mrl() then totem_action_play().
+ * For more information, see the documentation for totem_action_set_mrl_with_warning().
+ **/
 void
 totem_action_set_mrl_and_play (Totem *totem, const char *mrl, const char *subtitle)
 {
@@ -578,6 +621,18 @@ totem_action_load_media_device (Totem *totem, const char *device)
 	return retval;
 }
 
+/**
+ * totem_action_play_media_device:
+ * @totem: a #TotemObject
+ * @device: the media device's path
+ *
+ * Attempts to play the media device (for example, a DVD drive or CD drive)
+ * with the given @device path by first adding it to the playlist, then
+ * playing it.
+ *
+ * An error dialog will be displayed if Totem cannot read or play what's on
+ * the media device.
+ **/
 void
 totem_action_play_media_device (Totem *totem, const char *device)
 {
@@ -590,6 +645,17 @@ totem_action_play_media_device (Totem *totem, const char *device)
 	}
 }
 
+/**
+ * totem_action_play_media:
+ * @totem: a #TotemObject
+ * @type: the type of disc media
+ * @device: the media's device path
+ *
+ * Attempts to play the media found on @device (for example, a DVD in a drive or a DVB
+ * tuner) by first adding it to the playlist, then playing it.
+ *
+ * An error dialog will be displayed if Totem cannot support media of @type.
+ **/
 void
 totem_action_play_media (Totem *totem, TotemDiscMediaType type, const char *device)
 {
@@ -602,6 +668,12 @@ totem_action_play_media (Totem *totem, TotemDiscMediaType type, const char *devi
 	}
 }
 
+/**
+ * totem_action_stop:
+ * @totem: a #TotemObject
+ *
+ * Stops the current stream.
+ **/
 void
 totem_action_stop (Totem *totem)
 {
@@ -609,6 +681,13 @@ totem_action_stop (Totem *totem)
 	play_pause_set_label (totem, STATE_STOPPED);
 }
 
+/**
+ * totem_action_play_pause:
+ * @totem: a #TotemObject
+ *
+ * Gets the current MRL from the playlist and attempts to play it.
+ * If the stream is already playing, playback is paused.
+ **/
 void
 totem_action_play_pause (Totem *totem)
 {
@@ -639,6 +718,13 @@ totem_action_play_pause (Totem *totem)
 	}
 }
 
+/**
+ * totem_action_pause:
+ * @totem: a #TotemObject
+ *
+ * Pauses the current stream. If Totem is already paused, it continues
+ * to be paused.
+ **/
 void
 totem_action_pause (Totem *totem)
 {
@@ -697,6 +783,13 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 	return FALSE;
 }
 
+/**
+ * totem_action_fullscreen_toggle:
+ * @totem: a #TotemObject
+ *
+ * Toggles Totem's fullscreen state; if Totem is fullscreened, calling
+ * this makes it unfullscreened and vice-versa.
+ **/
 void
 totem_action_fullscreen_toggle (Totem *totem)
 {
@@ -706,6 +799,13 @@ totem_action_fullscreen_toggle (Totem *totem)
 		gtk_window_fullscreen (GTK_WINDOW (totem->win));
 }
 
+/**
+ * totem_action_fullscreen:
+ * @totem: a #TotemObject
+ * @state: %TRUE if Totem should be fullscreened
+ *
+ * Sets Totem's fullscreen state according to @state.
+ **/
 void
 totem_action_fullscreen (Totem *totem, gboolean state)
 {
@@ -903,6 +1003,24 @@ update_mrl_label (Totem *totem, const char *name)
 	}
 }
 
+/**
+ * totem_action_set_mrl_with_warning:
+ * @totem: a #TotemObject
+ * @mrl: the MRL to play
+ * @subtitle: a subtitle file to load, or %NULL
+ * @warn: %TRUE if error dialogs should be displayed
+ *
+ * Loads the specified @mrl and optionally the specified subtitle
+ * file. If @subtitle is %NULL Totem will attempt to auto-locate
+ * any subtitle files for @mrl.
+ *
+ * If a stream is already playing, it will be stopped and closed.
+ *
+ * If any errors are encountered, error dialogs will only be displayed
+ * if @warn is %TRUE.
+ *
+ * Return value: %TRUE on success
+ **/
 gboolean
 totem_action_set_mrl_with_warning (Totem *totem,
 				   const char *mrl, 
@@ -1022,6 +1140,17 @@ totem_action_set_mrl_with_warning (Totem *totem,
 	return retval;
 }
 
+/**
+ * totem_action_set_mrl:
+ * @totem: a #TotemObject
+ * @mrl: the MRL to load
+ * @subtitle: a subtitle file to load, or %NULL
+ *
+ * Calls totem_action_set_mrl_with_warning() with warnings enabled.
+ * For more information, see the documentation for totem_action_set_mrl_with_warning().
+ *
+ * Return value: %TRUE on success
+ **/
 gboolean
 totem_action_set_mrl (Totem *totem, const char *mrl, const char *subtitle)
 {
@@ -1072,12 +1201,27 @@ totem_action_direction (Totem *totem, TotemPlaylistDirection dir)
 	}
 }
 
+/**
+ * totem_action_previous:
+ * @totem: a #TotemObject
+ *
+ * If a DVD is being played, goes to the previous chapter. If a normal stream
+ * is being played, goes to the start of the stream if possible. If seeking is
+ * not possible, plays the previous entry in the playlist.
+ **/
 void
 totem_action_previous (Totem *totem)
 {
 	totem_action_direction (totem, TOTEM_PLAYLIST_DIRECTION_PREVIOUS);
 }
 
+/**
+ * totem_action_next:
+ * @totem: a #TotemObject
+ *
+ * If a DVD is being played, goes to the next chapter. If a normal stream
+ * is being played, plays the next entry in the playlist.
+ **/
 void
 totem_action_next (Totem *totem)
 {
@@ -1126,12 +1270,28 @@ totem_seek_time_rel (Totem *totem, gint64 time, gboolean relative)
 	}
 }
 
+/**
+ * totem_action_seek_relative:
+ * @totem: a #TotemObject
+ * @offset: the time offset to seek to
+ *
+ * Seeks to an @offset from the current position in the stream,
+ * or displays an error dialog if that's not possible.
+ **/
 void
 totem_action_seek_relative (Totem *totem, gint64 offset)
 {
 	totem_seek_time_rel (totem, offset, TRUE);
 }
 
+/**
+ * totem_action_seek_time:
+ * @totem: a #TotemObject
+ * @sec: the time to seek to
+ *
+ * Seeks to an absolute time in the stream, or displays an
+ * error dialog if that's not possible.
+ **/
 void
 totem_action_seek_time (Totem *totem, gint64 sec)
 {
@@ -1186,6 +1346,13 @@ totem_action_zoom_reset (Totem *totem)
 	totem_action_zoom (totem, 100);
 }
 
+/**
+ * totem_action_volume_relative:
+ * @totem: a #TotemObject
+ * @off_pct: the percentage by which to increase or decrease the volume
+ *
+ * Sets the volume relative to its current level.
+ **/
 void
 totem_action_volume_relative (Totem *totem, double off_pct)
 {
@@ -1198,6 +1365,13 @@ totem_action_volume_relative (Totem *totem, double off_pct)
 	bacon_video_widget_set_volume (totem->bvw, vol + off_pct);
 }
 
+/**
+ * totem_action_toggle_aspect_ratio:
+ * @totem: a #TotemObject
+ *
+ * Toggles the aspect ratio selected in the menu to the
+ * next one in the list.
+ **/
 void
 totem_action_toggle_aspect_ratio (Totem *totem)
 {
@@ -1213,18 +1387,42 @@ totem_action_toggle_aspect_ratio (Totem *totem)
 	gtk_radio_action_set_current_value (GTK_RADIO_ACTION (action), tmp);
 }
 
+/**
+ * totem_action_set_aspect_ratio:
+ * @totem: a #TotemObject
+ * @ratio: the aspect ratio to use
+ *
+ * Sets the aspect ratio selected in the menu to @ratio,
+ * as defined in #BaconVideoWidgetAspectRatio.
+ **/
 void
-totem_action_set_aspect_ratio (Totem *totem, int  ratio)
+totem_action_set_aspect_ratio (Totem *totem, int ratio)
 {
 	bacon_video_widget_set_aspect_ratio (totem->bvw, ratio);
 }
 
+/**
+ * totem_action_get_aspect_ratio:
+ * @totem: a #TotemObject
+ *
+ * Gets the current aspect ratio as defined in #BaconVideoWidgetAspectRatio.
+ *
+ * Return value: the current aspect ratio
+ **/
 int
 totem_action_get_aspect_ratio (Totem *totem)
 {
 	return (bacon_video_widget_get_aspect_ratio (totem->bvw));
 }
 
+/**
+ * totem_action_set_scale_ratio:
+ * @totem: a #TotemObject
+ * @ratio: the scale ratio to use
+ *
+ * Sets the video scale ratio, as a float where, for example,
+ * 1.0 is 1:1 and 2.0 is 2:1.
+ **/
 void
 totem_action_set_scale_ratio (Totem *totem, gfloat ratio)
 {
@@ -1400,17 +1598,34 @@ drag_video_cb (GtkWidget *widget,
 static void
 on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
 {
-	gchar *old_mrl, *new_mrl;
+	char *new_mrl;
 
-	old_mrl = totem_playlist_get_current_mrl (TOTEM_PLAYLIST (totem->playlist), NULL);
-	new_mrl = totem_pl_parser_resolve_url (old_mrl, mrl);
-	g_free (old_mrl);
+	if (strstr (mrl, "://") != NULL) {
+		new_mrl = NULL;
+	} else {
+		GFile *old_file, *parent, *new_file;
+		char *old_mrl;
+
+		/* Get the parent for the current MRL, that's our base */
+		old_mrl = totem_playlist_get_current_mrl (TOTEM_PLAYLIST (totem->playlist), NULL);
+		old_file = g_file_new_for_uri (old_mrl);
+		g_free (old_mrl);
+		parent = g_file_get_parent (old_file);
+		g_object_unref (old_file);
+
+		/* Resolve the URL */
+		new_file = g_file_get_child (parent, mrl);
+		g_object_unref (parent);
+
+		new_mrl = g_file_get_uri (new_file);
+		g_object_unref (new_file);
+	}
 
 	bacon_video_widget_close (totem->bvw);
 	totem_file_closed (totem);
 	totem_gdk_window_set_waiting_cursor (totem->win->window);
-	bacon_video_widget_open (totem->bvw, new_mrl, NULL);
-	totem_file_opened (totem, new_mrl);
+	bacon_video_widget_open (totem->bvw, new_mrl ? new_mrl : mrl, NULL);
+	totem_file_opened (totem, new_mrl ? new_mrl : mrl);
 	gdk_window_set_cursor (totem->win->window, NULL);
 	bacon_video_widget_play (bvw, NULL);
 	g_free (new_mrl);
@@ -1449,7 +1664,7 @@ on_channels_change_event (BaconVideoWidget *bvw, Totem *totem)
 static void
 on_playlist_change_name (TotemPlaylist *playlist, Totem *totem)
 {
-	char *name, *artist, *album, *title;
+	char *name;
 	gboolean cur;
 
 	if ((name = totem_playlist_get_current_title (playlist,
@@ -1457,15 +1672,6 @@ on_playlist_change_name (TotemPlaylist *playlist, Totem *totem)
 		update_mrl_label (totem, name);
 		g_free (name);
 	}
-
-	if (totem_playlist_get_current_metadata (playlist, &artist,
-						 &title, &album) != FALSE) {
-		totem_metadata_updated (totem, artist, title, album);
-
-		g_free (artist);
-		g_free (album);
-		g_free (title);
-	}
 }
 
 static void
@@ -1865,6 +2071,13 @@ show_controls (Totem *totem, gboolean was_fullscreen)
 	}
 }
 
+/**
+ * totem_action_toggle_controls:
+ * @totem: a #TotemObject
+ *
+ * If Totem's not fullscreened, this toggles the state of the "Show Controls"
+ * menu entry, and consequently shows or hides the controls in the UI.
+ **/
 void
 totem_action_toggle_controls (Totem *totem)
 {
@@ -1899,6 +2112,19 @@ totem_action_set_playlist_index (Totem *totem, guint index)
 	g_free (subtitle);
 }
 
+/**
+ * totem_action_remote:
+ * @totem: a #TotemObject
+ * @cmd: a #TotemRemoteCommand
+ * @url: an MRL to play, or %NULL
+ *
+ * Executes the specified @cmd on this instance of Totem. If @cmd
+ * is an operation requiring an MRL, @url is required; it can be %NULL
+ * otherwise.
+ *
+ * If Totem's fullscreened and the operation is executed correctly,
+ * the controls will appear as if the user had moved the mouse.
+ **/
 void
 totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 {
@@ -2196,6 +2422,14 @@ playlist_shuffle_toggle_cb (TotemPlaylist *playlist, gboolean shuffle, Totem *to
 			NULL, NULL, totem);
 }
 
+/**
+ * totem_is_fullscreen:
+ * @totem: a #TotemObject
+ *
+ * Returns %TRUE if Totem is fullscreened.
+ *
+ * Return value: %TRUE if Totem is fullscreened
+ **/
 gboolean
 totem_is_fullscreen (Totem *totem)
 {
@@ -2204,6 +2438,14 @@ totem_is_fullscreen (Totem *totem)
 	return (totem->controls_visibility == TOTEM_CONTROLS_FULLSCREEN);
 }
 
+/**
+ * totem_is_playing:
+ * @totem: a #TotemObject
+ *
+ * Returns %TRUE if Totem is playing a stream.
+ *
+ * Return value: %TRUE if Totem is playing a stream
+ **/
 gboolean
 totem_is_playing (Totem *totem)
 {
@@ -2223,6 +2465,14 @@ totem_is_paused (Totem *totem)
 	return totem->state == STATE_PAUSED;
 }
 
+/**
+ * totem_is_seekable:
+ * @totem: a #TotemObject
+ *
+ * Returns %TRUE if the current stream is seekable.
+ *
+ * Return value: %TRUE if the current stream is seekable
+ **/
 gboolean
 totem_is_seekable (Totem *totem)
 {
@@ -3138,10 +3388,6 @@ main (int argc, char **argv)
 		totem_action_error_and_exit (_("Totem could not initialize the configuration engine."), _("Make sure that GNOME is properly installed."), NULL);
 	}
 
-#ifndef HAVE_GTK_ONLY
-	gnome_authentication_manager_init ();
-#endif /* !HAVE_GTK_ONLY */
-
 	totem = g_object_new (TOTEM_TYPE_OBJECT, NULL);
 	totem->gc = gc;
 
